SystemOrganization addCategory: #'FLOdactic-Core'!

Object subclass: #AuthorsManager
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

AuthorsManager class
	instanceVariableNames: 'uniqueInstance'!

AuthorsManager class
	instanceVariableNames: 'uniqueInstance'!

!AuthorsManager class methodsFor: 'as yet unclassified' stamp: 'it 6/19/2013 21:52'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of TagsManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!AuthorsManager methodsFor: 'accessing' stamp: 'it 6/19/2013 21:59'!
add: anAuthor
	
	self validate: anAuthor.

	items add: anAuthor name -> anAuthor.
	Transcript show: 'Author doesnt exists on the system, adding it...'.
! !

!AuthorsManager methodsFor: 'accessing' stamp: 'it 6/19/2013 22:01'!
at: aNameString
	"comment stating purpose of message"

	^ items at: aNameString! !

!AuthorsManager methodsFor: 'testing' stamp: 'it 6/19/2013 22:01'!
includesKey: aNameString
	"comment stating purpose of message"

	^ items includesKey: aNameString! !

!AuthorsManager methodsFor: 'initialize-release' stamp: 'it 6/19/2013 22:01'!
initialize
	"Class responsability: Takes care of authors"
	
	super initialize.
	items := Dictionary new.
	Transcript show: 'AuthorsManager initialize'.
	! !

!AuthorsManager methodsFor: 'accessing' stamp: 'it 6/19/2013 21:52'!
items
	"Returns the internal structure"

	^ items.! !

!AuthorsManager methodsFor: 'initialize-release' stamp: 'it 6/19/2013 22:04'!
remove: aNameString
	items removeKey: aNameString ifAbsent: [
		self error: 'The author does not exists on the system, cannot remove it' ]! !

!AuthorsManager methodsFor: 'initialize-release' stamp: 'it 6/19/2013 21:52'!
removeAll
	"Internal method only. Not intended to be used as part of the main API"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].
! !

!AuthorsManager methodsFor: 'accessing' stamp: 'it 6/19/2013 21:52'!
select: aBlock
	"Performs a selection according to a code block"

	^ items select: aBlock.! !

!AuthorsManager methodsFor: 'testing' stamp: 'it 6/19/2013 22:06'!
validate: anAuthor	
	(anAuthor name)
		ifNil: [
			ValidationError signal: 'Author''s name cannot be empty!!'.
		] 
		ifNotNil: [
			(items includesKey: (anAuthor name))
				ifTrue: [
					(anAuthor ~= (items at: (anAuthor name)))
						ifTrue: [
							self error: 'Another author with name *', anAuthor name, '* already exists!!'.
						]
				]
		].
! !

Object subclass: #DidacticMaterial
	instanceVariableNames: 'authors contributors tags uri title created modified description type license status owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

DidacticMaterial class
	instanceVariableNames: 'owners'!

DidacticMaterial class
	instanceVariableNames: 'owners'!

!DidacticMaterial class methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 08:42'!
allLicenses
	^ #('Creative Commons' 'Open Publication' 'Open Content' 'GNU Public')! !

!DidacticMaterial class methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 08:41'!
allTypes
	^ #('Book' 'Internet Resource' 'Paper' 'Publication' 'Magazine' 'Picture' 'Presentation' 'Other')! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'it 6/19/2013 22:17'!
addAuthor: aNameString
	"Adds an Author name to the tags"

	| author  manager |
	manager := AuthorsManager singleton.
	
	"does it exist?"
	(manager includesKey: aNameString) ifTrue: [
		author := manager at: aNameString ]
	"else, add it to the manager"
	ifFalse: [
		author := Author new.
		author name: aNameString.
		manager add: aNameString -> author ].
	"finally add it to the authors object Dictionary itself"
	authors add: aNameString -> author.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:51'!
addContributor: aContributor
	"Adds a contributor to the contributors, granting with editor permissions"

	contributors add: aContributor name -> aContributor.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'it 6/19/2013 23:54'!
addContributorByUsername: aUsername
	"Adds a user to the contributors, granting with editor permissions"
	
	| user  manager |
	manager := UsersManager singleton.
	
	"does it exist?"
	(manager items includesKey: aUsername) ifTrue: [
		user := manager items at: aUsername.
		contributors add: aUsername name -> user ]! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'it 6/21/2013 09:20'!
addTag: aTagString
	"Adds aTagString to the tags"

	| tag testIncludedTag manager |
	manager := TagsManager singleton.
	
	testIncludedTag := manager includesKey: aTagString.
	"If it exists on the manager, get it"
	testIncludedTag ifTrue: [
		tag := manager at: aTagString.
	]
	"else, add it to the manager"
	ifFalse: [
		tag := TagObject new.
		tag name: aTagString.
		manager add: aTagString -> tag.
	].
	"finally add it to the tags object Dictionary itself"
	tags add: aTagString -> tag.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:52'!
authors
	"Returns the author"
	
	^ authors.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:22'!
catalogCard
	"Prints on the Transcript the catalog card of the didactic material"
	"Reference: http://www.biblioteca.bauru.unesp.br/?_escaped_fragment_=/servicos/ficha-catalografica/"
	| authorsData counter |

	authorsData := ''.	
	authors = nil ifFalse: [
		authors do: [ :author | Transcript show: author name; crlf ].
		Transcript tab; show: title, ', ', modified year; crlf.
		counter := 0.
		tags do: [ :tag | counter := counter+1.
			Transcript show: counter printString, '. ', tag name; space ].
		Transcript crlf.
		Transcript crlf.
	]
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'it 6/19/2013 23:27'!
contributors
	^ contributors! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:31'!
description
	^ description.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
description: aString 
	description := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doBlock
	"Blocks the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."
	
	status := 'blocked'.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doDisable
	"Disables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."
	
	status := 'disabled'.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:53'!
doEnable
	"Enables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."

	status := 'enabled'.
	self touch.! !

!DidacticMaterial methodsFor: 'initialize-release' stamp: 'it 6/21/2013 22:43'!
initialize
	"Class responsability: takes cares of a didactic material entity"
	super initialize.
	
	"No attribute is mandatory at this point, everything will be checked when it will be added"
	uri := nil.
	title := nil.
	description := nil.
	type := nil.
	license := nil.
	status := nil.
	
	"Agregations"
"	author := nil."
	contributors := Dictionary new.
	authors := Dictionary new.
	tags := Dictionary new.
"	owners := nil."
	
	
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:53'!
isBlocked
	"checks if the didactic material is blocked"
	
	^ status = 'blocked'! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:55'!
isDisabled
	"Checks if the didactic material is disabled"
	
	^ status = 'disabled'! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:55'!
isEnabled
	"Checks if the didactic material is enabled"
	
	^ status = 'enabled'! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:33'!
license
	^ license.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
license: aString 
	license := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/10/2013 18:04'!
matchesAuthorsByName: anExpression
	"Returns true if at least one of the authors name matches anExpression"

	| list |
	list := self selectAuthorsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByDescription: anExpression
	"Returns true if the didactic material description matches anExpression"

	^ anExpression match: description.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByOwnerEmail: anExpression
	"Returns true if the didactic material owner's email matches anExpression"

	^ anExpression match: (owner email).
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:58'!
matchesByOwnerName: anExpression
	"Returns true if the didactic material owner's name matches anExpression"

	^ anExpression match: (owner name).
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByStatus: anExpression
	"Returns true if the didactic material status matches anExpression"

	^ anExpression match: status.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByTitle: anExpression
	"Returns true if the didactic material title matches anExpression"

	^ anExpression match: title.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesByType: anExpression
	"Returns true if the didactic material type matches anExpression"

	^ anExpression match: type.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/11/2013 16:06'!
matchesContributorsByName: anExpression
	"Returns true if at least one of the contributors name matches anExpression"

	| list |
	list := self selectContributorsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 16:59'!
matchesContributorsByUsername: anExpression
	"Returns true if at least one of the contributors username matches anExpression"

	| list |
	list := self selectContributorsByUsername: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'testing' stamp: 'afv 5/12/2013 17:00'!
matchesTagByName: anExpression
	"Returns true if at least one of the tags name matches anExpression"

	| list |
	list := self selectTagsByName: anExpression.
	^ (list size) > 0.
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/10/2013 19:52'!
owner
	^ owner! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/7/2013 05:39'!
owner: anUser 
	| username |
	username := anUser username.

"	(owners includesKey: username) ifTrue: [
		msg := 'User *', username, '* is already an owner for that material'."
		"counter := items at: aTagString."
		"tags add: aTagString -> (counter + 1)."
"	].
	[(owners at: username)] ifError: [
		msg := 'User *', username, '* has been added as owner'.
		owners add: username -> anUser.
	].
	Transcript show: msg; cr."
	owner := anUser.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/10/2013 16:13'!
selectAuthorsByName: anExpression
	"Selects all the authors whose names matches anExpression"

"	^ authors select: [ :author | 
		author name ifNotNil: [
			anExpression match: (author name)
		]
		ifNil: [
			false.
		]
	]."

	^ authors select: [ :author | anExpression match: (author name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:56'!
selectContributorsByName: anExpression
	"Selects all the contributors whose names matches anExpression"

	^ contributors select: [ :contributor | anExpression match: (contributors name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:56'!
selectContributorsByUsername: anExpression
	"Selects all the contributor whose usernames matches anExpression"

	^ contributors select: [ :contributor | anExpression match: (contributor username)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/11/2013 16:51'!
selectTagsByName: anExpression
	"Selects tags whose names matches anExpression (could be partly)"

	^ tags select: [ :tag | anExpression match: (tag name)].
! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:33'!
status
	^ status.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:57'!
tags
	"Return the tags associated with the didactic materials as a Dictionary"

	^ tags.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:31'!
title
	^ title.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:40'!
title: aString 
	title := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:36'!
touch
	"Modifies the latest modification timestamp (similar to the touch *nix command)"

	modified := DateAndTime now.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:32'!
type
	^ type.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:34'!
type: aString 
	type := aString.
	self touch.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:29'!
uri
	^ uri.! !

!DidacticMaterial methodsFor: 'accessing' stamp: 'afv 5/1/2013 21:34'!
uri: aString 
	uri := aString.
	self touch.! !

Object subclass: #DidacticMaterialsManager
	instanceVariableNames: 'items tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

DidacticMaterialsManager class
	instanceVariableNames: 'uniqueInstance'!

DidacticMaterialsManager class
	instanceVariableNames: 'uniqueInstance'!

!DidacticMaterialsManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of DidacticMaterialManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/4/2013 11:56'!
addAll: aDictionary
	"Adds all the items on aDictionary to the internal structure"
	"TODO: check for possible errors (duplicated, missing attributes, etc)"

	items := aDictionary.! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:31'!
catalogCards
	"Prints on the Transcript the catalog cards of each didactic material"

	items do: [ :didacticMaterial | didacticMaterial catalogCard ].! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:02'!
initialize
	"Class responsability: handle didactic materials associations with other objects (authors, contributors, tags, etc)"
	
	super initialize.
	items := Dictionary new.
	Transcript show: 'DidacticMaterialManager initialize'; cr.! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'it 6/16/2013 12:52'!
items
	^ items! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 21:58'!
owner: anOwner add: aDidacticMaterial
	"Adds aDidacticMaterial to the internal structure. It also links that didactic material to anAuthor"
	self validate: aDidacticMaterial.
	
	Transcript show: 'The didactic material: *', aDidacticMaterial uri, '* DOESNT exists!!'; cr.
	aDidacticMaterial owner: anOwner.
	aDidacticMaterial doEnable.
	aDidacticMaterial touch.
	items add: aDidacticMaterial uri -> aDidacticMaterial.
	! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'it 6/17/2013 02:11'!
owner: anOwner edit: aDidacticMaterial
	"Edits aDidacticMaterial."
	| material |
	(items includesKey: aDidacticMaterial uri) ifFalse: [
		ValidationError signal: 'The material you are trying to update does not exist' ].
	
	self validate: aDidacticMaterial forExistingKey: aDidacticMaterial uri.
	
	material := items at: aDidacticMaterial uri.
	material copyFrom: aDidacticMaterial.

	Transcript show: 'Editing: *', aDidacticMaterial uri, '*'; cr.
	aDidacticMaterial touch.
	! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/10/2013 14:03'!
remove
	self removeAll.! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:35'!
remove: aDidacticMaterial
	"Removes from the structure anUriString. Raises errors on:
	- Non existing anUriString (verified with...)"
	
	| item test msg anUriString |
	anUriString := aDidacticMaterial uri.
	test := items includesKey: anUriString.
	(test) ifTrue: [
		msg := 'The didactic material exists on the system, removing it...'.
		Transcript show: msg; crlf.
		item := items at: anUriString.
		items removeKey: anUriString.
	].
	(test) ifFalse: [
		msg := 'The didactic material doesnt exists on the system, cannot remove it...'.
		Transcript show: msg; crlf.
		"self error: msg"

	].! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'afv 5/10/2013 14:56'!
removeAll
	"Our dirty solution to avoid persistance in Squeak. In order to have the tests passing, we need to remove all the items on each test. This is not intended to be used by the API user"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'it 6/16/2013 12:50'!
select: aFilter
	^ items select: aFilter! !

!DidacticMaterialsManager methodsFor: 'accessing' stamp: 'afv 5/4/2013 10:16'!
size
	"Returns the size of the internal structure"

	^ items size.! !

!DidacticMaterialsManager methodsFor: 'initialize-release' stamp: 'it 6/25/2013 02:09'!
toggleEnable: anItem
	"Enables or disables a Course"
	| item |
	(items includesKey: anItem uri) ifFalse: [
		ValidationError signal: 'The material you are trying to update does not exist' ].
	
	item := items at: anItem uri.

	item isEnabled
		ifTrue: [ item doDisable ]
		ifFalse: [ item doEnable ].

	Transcript show: 'Changing status of *', anItem title, '*'; cr.
	anItem touch.! !

!DidacticMaterialsManager methodsFor: 'testing' stamp: 'it 6/17/2013 02:16'!
validate: aDidacticMaterial
	"validates the material"
	self validate: aDidacticMaterial forExistingKey: nil! !

!DidacticMaterialsManager methodsFor: 'testing' stamp: 'afv 6/17/2013 15:15'!
validate: aDidacticMaterial forExistingKey: aURI
	"validates the material"
	
	(aDidacticMaterial uri isEmptyOrNil)
		ifTrue: [
			ValidationError signal: 'URI cannot be empty!!'.
		] 
		ifFalse: [
			(aURI ~= nil) ifTrue: [
				(items includesKey: (aDidacticMaterial uri))
					ifTrue: [
						"There is a DidacticMaterial with the same URI"
						(aDidacticMaterial uri ~= aURI)
							ifTrue: [
								ValidationError signal: 'Another didactic material with URI *', aDidacticMaterial uri, '* already exists!!'.
							]
					]
				]
		].

	(aDidacticMaterial title isEmptyOrNil) ifTrue: [
		ValidationError signal: 'Title cannot be empty!!'.
	].

	(aDidacticMaterial description isEmptyOrNil) ifTrue: [
		ValidationError signal: 'Description cannot be empty!!'.
	].

	(aDidacticMaterial license isEmptyOrNil) ifTrue: [
		ValidationError signal: 'License cannot be empty!!'.
	].

	(aDidacticMaterial type isEmptyOrNil) ifTrue: [
		ValidationError signal: 'Type cannot be empty!!'.
	].
! !

Object subclass: #FLOdacticFacade
	instanceVariableNames: 'users didacticMaterials tags loggedUser floogle search courses course01 subject01 class01A firstDay01 course02 subject02 class01B authors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

FLOdacticFacade class
	instanceVariableNames: 'uniqueInstance'!

FLOdacticFacade class
	instanceVariableNames: 'uniqueInstance'!

!FLOdacticFacade class methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 12:04'!
newTestsInstance
	^ self basicNew initializeWithoutSingletons.! !

!FLOdacticFacade class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:03'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of FLOdacticFacade...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/10/2013 20:13'!
addContributor: anUser didacticMaterial: aDidacticMaterial 
	aDidacticMaterial addContributor: anUser.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 6/18/2013 23:15'!
addCourse: aCourse
	"Adds a course. The owner will be the logged in user"
	
	| test |
	test := self loggedInOrRaiseErrorIfNot.
	test ifTrue: [
		aCourse touch.
		courses owner: loggedUser add: aCourse.
	]! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/10/2013 14:54'!
addMaterial: aDidacticMaterial 
	"Adds a didactic material. The owner will be the logged in user"
	
	| test |
	test := self loggedInOrRaiseErrorIfNot.
	test ifTrue: [
		aDidacticMaterial touch.
		didacticMaterials owner: loggedUser add: aDidacticMaterial.
	]! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 20:36'!
addUser: anUser 
	"Add an user
	This message doesn't needs login. This emulates self sign in to the system"
	
	users add: anUser.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/21/2013 08:45'!
allAuthors
	"Returns a copy of all tags"
	
	^ authors items.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/24/2013 23:37'!
allDidacticMaterials
	"Returns a copy of all didactic materials"
	
	^ didacticMaterials items.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/21/2013 08:29'!
allTags
	"Returns a copy of all tags"
	
	^ tags items.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/16/2013 01:31'!
allUsers
	"Returns all users"

	^ users items! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'it 6/21/2013 21:36'!
canDisable: anObject
	"Returns true only if loggedUser can disable anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isDidacticMaterial isUser |
	loggedUser ifNil: [ ^ false ].
	
	result := false.
	isDidacticMaterial := anObject class = DidacticMaterial.
	isDidacticMaterial ifTrue: [
		result := (loggedUser isAdmin) or: (loggedUser isModerator).
	].
	isUser := anObject class = User.
	isUser ifTrue: [
		result := loggedUser isAdmin.
	].
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'it 6/25/2013 02:26'!
canEdit: anObject
	"Returns true if loggedUser can edit anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			owner
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			owner
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isMaterial canDelete isCourse |
	loggedUser ifNil: [ ^ false ].
	result := false.
	canDelete := self canRemove: anObject. "handle errors"
	isMaterial := anObject class = DidacticMaterial.
	isMaterial ifTrue: [
		result := (anObject owner) username = loggedUser username.
		"Verify if loggedUser is a contributor"
		result ifFalse: [
			result := anObject matchesContributorsByUsername: (loggedUser username).
		].
	].
	isCourse := anObject isKindOf: CourseUnit.
	isCourse ifTrue: [
		result := (anObject owner) username = loggedUser username.
		"Verify if loggedUser is a contributor"
		result ifFalse: [
			result := anObject matchesContributorsByUsername: (loggedUser username).
		].
	].
	result := (result) or: (canDelete) or: (loggedUser isAdmin).
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'it 6/22/2013 09:15'!
canEnable: anObject
	"Returns true only if loggedUser can enable anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			administrators
		- an User instance in which case allowed user actors are:
			administrators
	Possible errors:
		User not logged in (nil)"

	| result |
	loggedUser ifNil: [ ^ false ].
	
	result := (loggedUser isAdmin) or: (loggedUser isModerator).
	^ result.! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'it 6/22/2013 14:11'!
canRemove: anObject
	"Returns true if loggedUser can remove anObject
	anObject can be:
		- a DidacticMaterial instance in which case allowed user actors are:
			owner
			moderators
			administrators
		- an User instance in which case allowed user actors are:
			owner (deletes it's own account)
			administrators
	Possible errors:
		User not logged in (nil)"

	| result isDidacticMaterial isUser isCourse |
	loggedUser ifNil: [ ^ false ].
	
	result := false.
	isDidacticMaterial := anObject class = DidacticMaterial.
	isDidacticMaterial ifTrue: [
		result := (anObject owner) username = loggedUser username.
		result ifFalse: [
			result := loggedUser isModerator.
		].
		result ifFalse: [
			result := loggedUser isAdmin.
		]
	].
	isCourse := anObject isKindOf: CourseUnit.
	isCourse ifTrue: [
		result := (anObject owner) username = loggedUser username.
		result ifFalse: [
			result := loggedUser isModerator.
		].
		result ifFalse: [
			result := loggedUser isAdmin.
		]
	].
	isUser := anObject class = User.
	isUser ifTrue: [
		result := (anObject username = loggedUser username) or: (loggedUser isAdmin).
	].
	^ result.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/15/2013 23:13'!
doLogin: aUser

	aUser
		ifNil: [ PermissionError signal: 'Please login with a valid user' ]
		ifNotNil: [ loggedUser := aUser ]
	
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 5/11/2013 19:42'!
doLogin: aUsernameString password: aPasswordString
	"Performs the login to the system.
	Returns true if login was successfull, false if not
	Possible errors:
	- Already logged in
	- Wrong username or username doesn't exist (delegated to UsersManager class)
	- Wrong password (delegated to UsersManager class)
	- User disabled (delegated to UsersManager class)
	"

	| result msg error |
	
	error := false.
	loggedUser ifNil: [
		loggedUser := users doLogin: aUsernameString password: aPasswordString.
		msg := 'Welcome back to FLOdactic', aUsernameString.
	]
	ifNotNil: [
		msg := 'Someone is already loggedin, logoff if this is not you!!'.
		error := true.
	].
	result := loggedUser username = aUsernameString.
	Transcript show: msg; cr.
	error ifTrue: [
		PermissionError signal: msg.
	].
	^ result.
	
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 21:15'!
doLogout
	"comment stating purpose of message"

	loggedUser := nil.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/22/2013 09:00'!
editCourse: aCourse
	self loggedInOrRaiseErrorIfNot.
	
	( self canEdit: aCourse ) ifFalse: [
		PermissionError signal: 'You do not have permission to update this.' ].

	courses owner: loggedUser edit: aCourse.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/22/2013 09:01'!
editDidacticMaterial: aDidacticMaterial 
	"Edit a didactic material. The owner must match the logged in user"

	self loggedInOrRaiseErrorIfNot.
	
	( self canEdit: aDidacticMaterial ) ifFalse: [
		PermissionError signal: 'You do not have permission to update this didactic material.' ].

	didacticMaterials owner: loggedUser edit: aDidacticMaterial.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/22/2013 09:12'!
editUser: aNewUser
	"If the username matches and the user is logged in then we accept aNewUser as the updated object
	This is the best way we came arround to allow updates, we don't feel it would be nice to pass every
	property on the same message, so we pass a full object."

	self loggedInOrRaiseErrorIfNot.
	
	( self canEdit: aNewUser ) ifFalse: [
		PermissionError signal: 'You do not have permission to update this didactic material.' ].

	users save: aNewUser! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/11/2013 22:12'!
feedbackBlock: anObject
	"Changes the status of anObject to locked so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	anObject doBlock.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 00:57'!
feedbackDisable: anObject
	"Changes the status of anObject to disabled so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	(self canDisable: anObject) ifTrue: [
		anObject doDisable.
	].! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 01:01'!
feedbackEnable: anObject
	"Changes the status of anObject to enabled so admin or moderators can review the object (user
	or didactic material), and if so block it or not"
	
	(self canEnable: anObject) ifTrue: [
		anObject doEnable.
	].! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 6/20/2013 10:24'!
initialize
	"Class responsability: Abstract a session of a user on the FLOdactic system and centralize all user's actions on the system within a single access interface"

	users := UsersManager singleton.
	didacticMaterials := DidacticMaterialsManager singleton.
	tags := TagsManager singleton.
	courses := CoursesManager singleton.
	authors := AuthorsManager singleton.
	
	search := SearchEngine new.
	
	Transcript show: 'Creating users interface...'; crlf.
	
	"self populateWithTestData."
	Transcript show: 'Adding some bogus data...'; crlf.
	
	^ self! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'it 5/11/2013 21:23'!
initializeWithoutSingletons
	"This is a method only for use with the tests suite, in order to recreate objects all the time"

	users := UsersManager new.
	didacticMaterials := DidacticMaterialsManager new.
	tags := TagsManager new.
	
	search := SearchEngine new.
	"Need to set the same Managers into the SearchEngine, otherwise tests will fail"
	search didacticMaterialsManager: didacticMaterials.
	search usersManager: users.
	search tagsManager: tags.
	
	Transcript show: 'Creating users interface...'; crlf.
	^ self! !

!FLOdacticFacade methodsFor: 'testing' stamp: 'it 6/17/2013 00:59'!
loggedInOrRaiseErrorIfNot
	"Checks if the user is logged in, and if not raises an error"

	| msg test |
	test := loggedUser = nil.
	test ifTrue: [ 
		msg := 'Not logged in, you must log in first in order to complete that action. No cookies for you my friend!!'.
		PermissionError signal: msg
	].
	^ test not.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 16:36'!
loggedUser
	"Returns the logged user"
	
	^ loggedUser veryDeepCopy.! !

!FLOdacticFacade methodsFor: 'data-population' stamp: 'afv 6/25/2013 10:22'!
populateWithTestData

	| admin01 albertAuthor albertUser igorUser moderator01 moderator02 material01 material02 subject03 |
	
	albertAuthor := Author new.
	albertAuthor name: 'Albert De La Fuente'.
	
	"Create special users objects"
	Transcript show: 'Creating users...'; crlf.
	admin01 := User new.
	moderator01 := User new.
	moderator02 := User new.
	
	admin01 username: 'admin01'; password: 'q'; name: 'Leonardo Santos'; email: 'admintest@flodactic.org'; withAdminPrivileges; doEnable.
	moderator01 username: 'mod01'; password: 'q'; name: 'Rodolpho Bustamante'; email: 'mod01test@flodactic.org'; withModeratorPrivileges; doEnable.
	moderator02 username: 'mod02'; password: 'q'; name: 'Samuel Rodrigues'; email: 'mod02test@flodactic.org'; withModeratorPrivileges; doEnable.
	
	"Create regular users objects"
	albertUser := User new.
	igorUser := User new.
	
	albertUser username: 'albert'; password: 'q'; name: 'Albert De La Fuente'; email: 'albert@ime.usp.br'; withUserPrivileges; doEnable.
	igorUser username: 'topcin'; password: 'q'; name: 'Igor Topcin'; email: 'topcin@ime.usp.br'; withUserPrivileges; doEnable.
	
	material01 := DidacticMaterial new.
	material02 := DidacticMaterial new.
	
	material01 uri: 'http://www.albertdelafuente.com/doku.php/wiki/devel/python/ldap-auth';
		title: 'Python LDAP tutorial';
		description: 'A tutorial on how to use an LDAP server on Python';
		type: 'Internet Resource';
		license: 'Creative Commons';
		addTag: 'Python';
		addTag: 'Tutorial';
		addTag: 'LDAP';
		doEnable..

	material02 uri: 'http://www.albertdelafuente.com/doku.php/wiki/devel/python/whatsapp-api';
		title: 'Python Whatsapp tutorial';
		description: 'A tutorial on how to use an Whatsapp API on Python';
		type: 'Internet Resource';
		license: 'Creative Commons';
		addTag: 'Python';
		addTag: 'Tutorial';
		addTag: 'Whatsapp';
		doEnable..
		
	material01 addAuthor: 'Albert'.
	material02 addAuthor: 'Albert'.

	"USERS POPULATION"
		
	self addUser: admin01.
	self addUser: moderator01.
	self addUser: moderator02.
	
	"LOGIN AS ADMIN01 AND POPULATE DATA"
	self doLogin: 'admin01' password: 'q'.
	
	self addUser: albertUser.
	self addUser: igorUser.
	
	"MATERIALS POPULATION"
	
	self addMaterial: material01.
	self addMaterial: material02.
	
	"COURSES POPULATION"
	
	course01 := CourseCourse new.
	course01 initializeWithCode: 'MAC' initializeWithName: 'B.S. Computer Science' creator: admin01.
"		addTag: 'Python';
		addTag: 'Tutorial';
		addTag: 'Whatsapp'."
	course01 title: 'B.S. Computer Science'.
	course01 description: 'B.S. C.S.'; doEnable..
	
	course02 := CourseCourse new.
	course02 initializeWithCode: 'MAT' initializeWithName: 'B.S. Mathematics' creator: admin01.
	course02 title: 'B.S. Mathematics'.
	course02 description: 'B.S. C.S.'; doEnable..

	"SUBJECTS POPULATION"

	subject01 := CourseSubject new.
	subject01 initializeWithCode: 'MAC5777' initializeWithName: 'Cool stuff in Python' creator: admin01.
	subject01 parent: course01.
	subject01 description: 'Test'; doEnable..
	course01 children add: subject01 code -> subject01.
	
	subject02 := CourseSubject new.
	subject02 initializeWithCode: 'MAC5888' initializeWithName: 'N-plane complex futile stuff' creator: admin01.
	subject02 parent: course02.
	subject02 description: 'Test'; doEnable..
	course02 children add: subject02 code -> subject02.

	subject03 := CourseSubject new.
	subject03 initializeWithCode: 'MAC5999' initializeWithName: 'Best subject ever' creator: admin01.
	subject03 parent: course02.
	subject03 description: 'Test'; doEnable..
	course02 children add: subject03 code -> subject03.
	
	"CLASSES POPULATION"

	class01A := CourseClass new.
	class01A initializeWithCode: 'MAC5777-A' initializeWithName: 'Class A' creator: admin01.
	class01A parent: subject01.
	class01A description: 'Test'; doEnable..
	subject01 children add: class01A code -> class01A.
	
	class01B := CourseClass new.
	class01B initializeWithCode: 'MAC5777-B' initializeWithName: 'Class B' creator: admin01.
	class01B parent: subject01.
	class01B description: 'Test'; doEnable..
	subject01 children add: class01B code -> class01B.
	
	firstDay01 := CourseLesson new.
	firstDay01 initializeWithCode: 'MAC5777-A-01' initializeWithName: 'Python LDAP Auth' creator: admin01.
	firstDay01 parent: class01A.
	firstDay01 description: 'Test'; doEnable..
	class01A children add: firstDay01 code -> firstDay01.

	"FACADEPOPULATION"
	
	self addCourse: course01.
	self addCourse: course02.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/22/2013 13:43'!
refreshCourseObject: aCourse
	^ courses findActualItem: aCourse.
! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/9/2013 19:54'!
remove
	search remove.
	
	didacticMaterials remove.
	users remove.
	tags remove.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 6/20/2013 10:23'!
removeAll
	"This is for testing purposes, persistent objects within tests are breaking some of the tests"

	courses ifNotNil: [
		courses removeAll.
		courses release.
	].

	users ifNotNil: [
		users removeAll.
		users release.
	].

	authors ifNotNil: [
		authors removeAll.
		authors release.
	].
	
	didacticMaterials removeAll.
	didacticMaterials release.

	tags removeAll.
	tags release.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'it 6/21/2013 17:17'!
removeCourse: aCourse
	(self canRemove: aCourse) ifTrue: [
		courses remove: aCourse.
	].! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:34'!
removeDidacticMaterial: aDidacticMaterial
	(self canRemove: aDidacticMaterial) ifTrue: [
		didacticMaterials remove: aDidacticMaterial.
	].! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 22:28'!
removeUser: anUsernameString 
	users remove: anUsernameString.! !

!FLOdacticFacade methodsFor: 'initialize-release' stamp: 'afv 5/11/2013 12:23'!
resetInstance
	"We couldn't sucessfuly delete objects in Squek so we reset the instance and it's attributes.
	Unelegant, we know, but we couln't find other better way"

	users removeAll..
	didacticMaterials removeAll.
	tags removeAll.
	
	users := nil.
	didacticMaterials := nil.
	tags := nil.
	search := nil.
	
	Transcript show: 'Clearing users interface...'; crlf.
	^ self! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 6/17/2013 15:32'!
searchCourses: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression,
	by default only enabled documents will be shown"

	| results filter |
	results := search searchCourses: aStringExpression.
	((loggedUser isAdmin) or: (loggedUser isModerator)) ifTrue: [	
		filter := search filterAll.
	]
	ifFalse: [
		filter := Filter coursesByStatus: 'enabled'. "TODO!!"
	].
	results := search addFilter: filter.
	^ results! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 6/8/2013 10:22'!
searchDidacticMaterials: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression,
	by default only enabled documents will be shown"

	| results filter |
	results := search searchDidacticMaterials: aStringExpression.
	((loggedUser isAdmin) or: (loggedUser isModerator)) ifTrue: [	
		filter := search filterAll.
	]
	ifFalse: [
		filter := Filter didacticMaterialsByStatus: 'enabled'.
	].
	results := search addFilter: filter.
	^ results! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 5/11/2013 14:41'!
searchTags: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression"

	^ search searchTags: aStringExpression.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/9/2013 22:13'!
searchUsers: aStringExpression
	"Returns an ordered collection with the search results by aStringExpression"

	^ search searchUsers: aStringExpression.! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/22/2013 12:26'!
toggleCourseStatus: aCourse
	self loggedInOrRaiseErrorIfNot.
	
	( self canEnable: aCourse ) ifFalse: [
		PermissionError signal: 'You do not have permission to enable this.' ].

	courses toggleEnable: aCourse.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'it 6/25/2013 02:05'!
toggleMaterialStatus: aMaterial
	self loggedInOrRaiseErrorIfNot.
	
	( self canEnable: aMaterial ) ifFalse: [
		PermissionError signal: 'You do not have permission to enable this.' ].

	didacticMaterials toggleEnable: aMaterial.
! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/12/2013 14:31'!
transcriptCatalogCards
	"Prints on the transcript a catalog card for each didactic material"

	didacticMaterials catalogCards.
"	didacticMaterials do: [ :didacticMaterial | didacticMaterial catalogCard ]."
	! !

!FLOdacticFacade methodsFor: 'accessing' stamp: 'afv 5/2/2013 18:03'!
usersAsOrderedCollection
	^ users asOrderedCollection.! !

Object subclass: #Filter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!Filter class methodsFor: 'filtering' stamp: 'afv 6/17/2013 15:27'!
coursesAll
	"This filter expression doesn't filter anything, it will be used to return the results unmodified on the admin / moderators searches"
	| |

	^ [ :course | true].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/22/2013 16:54'!
coursesByStatus: aStatusString
	"Filter expression by status"

	| searchExpression |
	self isSearchingAString: aStatusString.
	searchExpression := self defaultFilterPolicy: aStatusString.
	^ [ :didacticMaterial | didacticMaterial matchesByStatus: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 09:51'!
defaultFilterPolicy: anExpressionString
	"This is the default policy for searching message
	We decided to wrap the expression with wildcards as default, this can be easily changed here"

	^ self wildCardExpression: anExpressionString.! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:25'!
didacticMaterialsAll
	"This filter expression doesn't filter anything, it will be used to return the results unmodified on the admin / moderators searches"
	| |

	^ [ :didacticMaterial | true].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:11'!
didacticMaterialsByAuthorName: anAuthorNameString
	"Filter expression by Author's name"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: anAuthorNameString.
	searchExpression := self defaultFilterPolicy: anAuthorNameString.
	^ [ :didacticMaterial | didacticMaterial matchesAuthorsByName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByDescription: aDescriptionString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aDescriptionString.
	searchExpression := self defaultFilterPolicy: aDescriptionString.
	^ [ :didacticMaterial | didacticMaterial matchesByDescription: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByOwnerEmail: anOwnerString
	"Didactic material filter expression by owner"
	| searchExpression |
	
	self isSearchingAString: anOwnerString.
	searchExpression := self defaultFilterPolicy: anOwnerString.
	^ [ :didacticMaterial | didacticMaterial matchesByOwnerEmail: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByOwnerName: anOwnerString
	"Didactic material filter expression by owner"
	| searchExpression |
	
	self isSearchingAString: anOwnerString.
	searchExpression := self defaultFilterPolicy: anOwnerString.
	^ [ :didacticMaterial | didacticMaterial matchesByOwnerName: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:07'!
didacticMaterialsByStatus: aStatusString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aStatusString.
	searchExpression := self defaultFilterPolicy: aStatusString.
	^ [ :didacticMaterial | didacticMaterial matchesByStatus: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByTag: aTagString
	"Didactic material filter expression by Author's name"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTagString.
	searchExpression := self defaultFilterPolicy: aTagString.
	^ [ :didacticMaterial | didacticMaterial matchesTagByName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByTitle: aTitleString
	"Filter expression by Title"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTitleString.
	searchExpression := self defaultFilterPolicy: aTitleString.
	^ [ :didacticMaterial | didacticMaterial matchesByTitle: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:06'!
didacticMaterialsByType: aTypeString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTypeString.
	searchExpression := self defaultFilterPolicy: aTypeString.
	^ [ :didacticMaterial | didacticMaterial matchesByType: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/8/2013 10:23'!
isSearchingAString: input
	"Checks if the filter's input parameter is null/empty and raises an error if so"
	| isInvalid |
	(isInvalid := input isString not) ifFalse: [
		isInvalid := input isEmpty
	].

	isInvalid ifTrue: [self error: 'Invalid filter expression. It must be a string'].
	^ isInvalid! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/22/2013 17:09'!
objectByStatus: aStatusString
	"Filter expression by status"

	| searchExpression |
	self isSearchingAString: aStatusString.
	searchExpression := self defaultFilterPolicy: aStatusString.
	^ [ :object | object matchesByStatus: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/22/2013 17:23'!
objectsAll
	"This filter expression doesn't filter anything, it will be used to return the results unmodified on the admin / moderators searches"
	| |

	^ [ :object | true].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/23/2013 02:00'!
objectsByOwnerName: anOwnerString
	"Didactic material filter expression by owner"
	| searchExpression |
	
	self isSearchingAString: anOwnerString.
	searchExpression := self defaultFilterPolicy: anOwnerString.
	^ [ :object | object matchesByOwnerName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/23/2013 02:00'!
objectsByTag: aTagString
	"Didactic material filter expression by Author's name"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| searchExpression |
	self isSearchingAString: aTagString.
	searchExpression := self defaultFilterPolicy: aTagString.
	^ [ :object | object matchesTagByName: searchExpression].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/22/2013 17:08'!
subjectsByStatus
	! !

!Filter class methodsFor: 'filtering' stamp: 'it 6/16/2013 13:13'!
usersAll
	^ [ :user | true].! !

!Filter class methodsFor: 'filtering' stamp: 'afv 6/8/2013 10:05'!
usersByRole: aRoleString
	"Users filtered by type"
	| searchExpression |
	
	self isSearchingAString: aRoleString.
	searchExpression := self defaultFilterPolicy: aRoleString.
	^ [ :user | user matchesByRole: searchExpression].! !

!Filter class methodsFor: 'as yet unclassified' stamp: 'afv 6/7/2013 12:11'!
wildCardExpression: anExpression
	"Returns an expression surrounded by asterisk (wildcard search)"

	^ '*', anExpression, '*'. "let's put some wildcards on it"! !

Object subclass: #Person
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

Person subclass: #Author
	instanceVariableNames: 'id bio'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!Author methodsFor: 'accessing' stamp: 'afv 5/2/2013 20:32'!
bio
	^ bio.! !

!Author methodsFor: 'accessing' stamp: 'afv 5/2/2013 20:32'!
bio: aString
	bio := aString.! !

!Author methodsFor: 'accessing' stamp: 'it 6/19/2013 21:59'!
initialize
	"This class takes cares of an author entity"
	super initialize.
	
	name := nil.
	bio := nil.! !

!Person methodsFor: 'initialize-release' stamp: 'it 6/19/2013 21:59'!
initialize
	"Class responsability: Refactored common class from User and Author. Takes care of common fields"

	super initialize.
	name := nil.! !

!Person methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:08'!
name
	^ name.! !

!Person methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:41'!
name: aString 
	name := aString.! !

Person subclass: #User
	instanceVariableNames: 'username password email role status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!User commentStamp: 'afv 5/1/2013 20:44' prior: 0!
User class

Responsability: Represent a single user and its attributes!

!User methodsFor: 'accessing' stamp: 'afv 5/11/2013 22:09'!
doBlock
	status := 'blocked'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 17:43'!
doDisable
	status := 'disabled'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 17:42'!
doEnable
	status := 'enabled'.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:00'!
email
	^ email! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:41'!
email: aString 
	email := aString.! !

!User methodsFor: 'testing' stamp: 'it 6/14/2013 00:44'!
hasEmptyPassword
	"comment stating purpose of message"

	^ password isEmptyOrNil.! !

!User methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:07'!
initialize
	"Class responsability: handles a single user"
	
	super initialize.
	
	"Mandatory user's attributes"
	username := nil.
	password := nil.
	email := nil.
	
	"Automatic attributes (defined by the manager)"
	role := nil.
	status := nil.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 17:38'!
isAdmin
	^ role = 'administrator'.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 22:10'!
isBlocked
	^ status = 'blocked'.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 22:10'!
isDisabled
	^ status = 'disabled'.! !

!User methodsFor: 'testing' stamp: 'afv 5/2/2013 18:31'!
isEnabled
	^ status = 'enabled'.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 17:38'!
isModerator
	^ role = 'moderator'.! !

!User methodsFor: 'testing' stamp: 'afv 5/10/2013 18:28'!
isUser
	^ role = 'user'.! !

!User methodsFor: 'testing' stamp: 'afv 5/2/2013 19:23'!
matchUsername: aUsernameString matchPassword: aPasswordString 
	^ username = aUsernameString and: password = aPasswordString.! !

!User methodsFor: 'testing' stamp: 'afv 5/11/2013 21:17'!
matchesByRole: anExpression
	"Returns true if the didactic material' description matches anExpression"

	^ anExpression match: role.
! !

!User methodsFor: 'accessing' stamp: 'it 6/13/2013 22:19'!
password 
	^ password! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:40'!
password: aString 
	password := aString.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 10:59'!
release
	"Not sure if this is right but we didn't find any way to actualy use the release message in a way that works as we think it should.
	Perhaps this is not a good practice but we intentionally throw the reference to that object and the garbage collector should
	do the rest of the trics (hopefully)"

	self initialize.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:30'!
role
	"comment stating purpose of message"

	^ role.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 22:01'!
status
	^ status.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 15:39'!
username
	^ username.! !

!User methodsFor: 'accessing' stamp: 'afv 4/30/2013 13:39'!
username: aString 
	username := aString.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:16'!
withAdminPrivileges
	role := 'administrator'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:16'!
withModeratorPrivileges
	role := 'moderator'.! !

!User methodsFor: 'accessing' stamp: 'afv 5/10/2013 18:17'!
withUserPrivileges
	role := 'user'.! !

Object subclass: #SearchEngine
	instanceVariableNames: 'didacticMaterialsManager resultsByTitle resultsByDescription searchResults usersManager tagsManager coursesManager authorsManager'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!SearchEngine methodsFor: 'accessing' stamp: 'it 6/16/2013 13:19'!
addFilter: aFilter
	"Applies aFilter to the searchResults and returns the new results OrderedCollection"

	| oldResults results test |
	test := self checkNullResults.
	test ifFalse: [
		oldResults := searchResults veryDeepCopy.
		results := oldResults select: aFilter.
	].
	^ results.
"Do not uncomment the following line. The search screens assume the searchResults are never modified when a filter is added."
"	searchResults := results"
	! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 6/19/2013 22:07'!
authorsManager: anAuthorsManager
	"Internal API usage only."
	
	authorsManager := anAuthorsManager! !

!SearchEngine methodsFor: 'testing' stamp: 'it 5/11/2013 13:51'!
checkNullResults
	"comment stating purpose of message"

	| test |
	(test := searchResults = nil) ifTrue: [
		self error: 'You must first search and then add filters to the results.'
	].
	^ test! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
didacticMaterialsManager: aDidacticMaterialsManager
	"Internal API usage only."
	
	didacticMaterialsManager := aDidacticMaterialsManager! !

!SearchEngine methodsFor: 'filtering' stamp: 'it 6/16/2013 12:45'!
filterbyDidacticMaterialsByStatus: aStatusString
	"Filter expression by description"

	"okay here's another crazy idea... this should return a block that acts as a filter,
	so the filter strategy is a generic algorithm with just 'plugins' of code!!"
	| test searchExpression |
	"self checkNullSearch: aStatusString."
	test := self checkNullResults.
	test ifFalse: [
		searchExpression := self wildCardExpression: aStatusString.
		^ [ :didacticMaterial | didacticMaterial matchesByStatus: searchExpression].
	].! !

!SearchEngine methodsFor: 'initialize-release' stamp: 'it 6/19/2013 22:08'!
initialize
	"Class responsability: Takes care of searches on the DidacticMaterialsManager, TagsManager and UsersManager singletons instances"
	
	super initialize.
	didacticMaterialsManager := DidacticMaterialsManager singleton.
	usersManager := UsersManager singleton.
	tagsManager := TagsManager singleton.
	coursesManager := CoursesManager singleton.
	authorsManager := AuthorsManager singleton.
	
	searchResults := nil.! !

!SearchEngine methodsFor: 'searching' stamp: 'it 6/19/2013 22:09'!
searchAuthors: aStringExpression
	"Searches for Authors that match the following criteria:
	- name matches the informed search string aStringExpression"
		
	| searchExpression |
	
	searchResults := nil.
	
	(aStringExpression isEmptyOrNil)
		ifFalse: [
			searchExpression := self wildCardExpression: aStringExpression.
			searchResults := authorsManager select: [ :author | (searchExpression match: author name) ] ]
		ifTrue: [
			searchResults := authorsManager items ].
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'it 6/22/2013 10:57'!
searchCourses: aStringExpression
	^ self searchCourses: aStringExpression in: nil! !

!SearchEngine methodsFor: 'searching' stamp: 'afv 6/23/2013 00:35'!
searchCourses: aStringExpression in: aCourseUnit
	| collection searchExpression |
	aCourseUnit
		ifNil: [ collection := coursesManager items ]
		ifNotNil: [ collection := aCourseUnit children ].
	
	(aStringExpression isEmptyOrNil)
		ifFalse: [
			searchExpression := self wildCardExpression: aStringExpression.
			Transcript show: searchExpression; cr.
			searchResults := collection select: [ :item |
				Transcript show: item title; cr.
				Transcript show: item description; cr.

				(searchExpression match: item code) or:
				(searchExpression match: item title) or:
				(searchExpression match: item description) ] ]
		ifTrue: [
			searchResults := collection ].
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'it 6/16/2013 12:48'!
searchDidacticMaterials: aStringExpression
	"Returns an ordered collection of foun items by aStringExpression by all fields
	Later filters can by applied to narrow search results (as with google for instance)"

	| searchExpression |
	
	searchResults := nil.	
	(aStringExpression isEmptyOrNil)
		ifFalse: [
			searchExpression := self wildCardExpression: aStringExpression.
			searchResults := didacticMaterialsManager select: [
				:didacticMaterial |
				(searchExpression match: didacticMaterial title) or:
				(searchExpression match: didacticMaterial description)
			] ]
		ifTrue: [
			searchResults := didacticMaterialsManager items ].
		
"		resultsByTitle := (didacticMaterialsManager select: [:didacticMaterial | (searchExpression match: didacticMaterial title)]) items.
		resultsByDescription := (didacticMaterialsManager select: [:didacticMaterial | (searchExpression match: didacticMaterial description)]) items."
"		searchResults := (resultsByTitle asOrderedCollection) addAll: (resultsByDescription asOrderedCollection).."
	^ searchResults! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 6/16/2013 13:10'!
searchResults
	"Convenience method for returning the last search results (i.e. the returned value from a #search*# call). Note that added filters are not taken into consideration by this method."
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'it 6/16/2013 12:55'!
searchTags: aStringExpression
	"Searches for Tags that match the following criteria:
	- name matches the informed search string aStringExpression"
		
	| searchExpression |
	
	searchResults := nil.
	
	(aStringExpression isEmptyOrNil)
		ifFalse: [
			searchExpression := self wildCardExpression: aStringExpression.
			searchResults := tagsManager select: [ :tag | (searchExpression match: tag name) ] ]
		ifTrue: [
			searchResults := tagsManager items ].
	^ searchResults! !

!SearchEngine methodsFor: 'searching' stamp: 'it 6/16/2013 12:56'!
searchUsers: aStringExpression
	"Searches for Users that match the following criteria:
	- name matches the informed search string aStringExpression OR
	- username matches aStringExpression OR
	- email matches aStringExpression"
	
	| searchExpression |
	
	searchResults := nil.
	
	(aStringExpression isEmptyOrNil)
		ifFalse: [
			searchExpression := self wildCardExpression: aStringExpression.
			searchResults := usersManager select: [
				:user |
				(searchExpression match: user name) or:
				(searchExpression match: user username) or:
				(searchExpression match: user email)
			] ]
		ifTrue: [
			searchResults := usersManager items ].

	^ searchResults! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
tagsManager: aTagsManager
	"Internal API usage only."
	
	tagsManager := aTagsManager! !

!SearchEngine methodsFor: 'accessing' stamp: 'it 5/11/2013 21:20'!
usersManager: aUsersManager
	"Internal API usage only."
	
	usersManager := aUsersManager! !

!SearchEngine methodsFor: 'accessing' stamp: 'afv 5/6/2013 20:50'!
wildCardExpression: anExpression
	"Returns an expression surrounded by asterisk (wildcard search)"

	^ '*', anExpression, '*'. "let's put some wildcards on it"! !

Object subclass: #TagObject
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

!TagObject methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 17:12'!
name
	"comment stating purpose of message"

	^ name.! !

!TagObject methodsFor: 'as yet unclassified' stamp: 'afv 5/11/2013 17:11'!
name: aString
	"comment stating purpose of message"

	name := aString.! !

Object subclass: #TagsManager
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

TagsManager class
	instanceVariableNames: 'uniqueInstance'!

!TagsManager commentStamp: 'afv 5/1/2013 20:44' prior: 0!
TagsManager class

Respnsability: Manage all the tags within the system.

We used a Dictionary as a support structure to store the Tag objects. We choose to use a Dictionary instead of a Set since we pretend to dynamically update the count on DidacticMaterial associated with each tag, so we can have an idea on popular Tags.!

TagsManager class
	instanceVariableNames: 'uniqueInstance'!

!TagsManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of TagsManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 00:15'!
add: aTag
	
	self validate: aTag.

	items add: aTag name -> aTag.
	Transcript show: 'Tag doesnt exists on the system, adding it...'.
! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/10/2013 23:40'!
asOrderedCollection
	"Returns an ordered collection containing all tags"

	^ items asOrderedCollection.! !

!TagsManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 15:45'!
at: aTagString.
	"comment stating purpose of message"

	^ items at: aTagString.! !

!TagsManager methodsFor: 'testing' stamp: 'afv 5/11/2013 15:48'!
includesKey: aTagString
	"comment stating purpose of message"

	^ items includesKey: aTagString.! !

!TagsManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:06'!
initialize
	"Class responsability: Takes care of tags"
	
	super initialize.
	items := Dictionary new.
	Transcript show: 'TagsManager initialize'.
	! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 00:28'!
items
	"Returns the internal structure"

	^ items.! !

!TagsManager methodsFor: 'initialize-release' stamp: 'it 5/11/2013 00:40'!
remove: aTagNameString
	"Removes from the structure aTagNameString. Raises errors if:
	- Tag does not exist"
	
	items removeKey: aTagNameString ifAbsent: [
		self error: 'The tag does not exists on the system, cannot remove it...'.
	].
! !

!TagsManager methodsFor: 'initialize-release' stamp: 'it 5/10/2013 23:46'!
removeAll
	"Internal method only. Not intended to be used as part of the main API"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].
! !

!TagsManager methodsFor: 'accessing' stamp: 'it 5/11/2013 18:37'!
select: aBlock
	"Performs a selection according to a code block"

	^ items select: aBlock.! !

!TagsManager methodsFor: 'testing' stamp: 'it 5/11/2013 00:13'!
validate: aTag
	"validates the tag, raising an error if:
	- name is empty
	- name is already being used by another tag"
	
	(aTag name)
		ifNil: [
			self error: 'Tag name cannot be empty!!'.
		] 
		ifNotNil: [
			(items includesKey: (aTag name))
				ifTrue: [
					"There is a Tag with the same name"
					(aTag ~= (items at: (aTag name)))
						ifTrue: [
							self error: 'Another tag with name *', aTag name, '* already exists!!'.
						]
				]
		].
! !

Object subclass: #UsersManager
	instanceVariableNames: 'items emailsMap emailsSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

UsersManager class
	instanceVariableNames: 'uniqueInstance'!

!UsersManager commentStamp: 'afv 5/1/2013 20:47' prior: 0!
UsersManager class

Responsability: Manage all the users on the system.

Restrictions:
	- Username cannot be empty (verified with testEmptyUsername)
	- Password cannot be empty (verified with testEmptyPassword)
	- Email cannot be empty (verified with testEmptyEmail)
	- Role cannot be empty (verified with testEmptyRole)
	- Status cannot be empty (verified with testEmptyStatus)
	- Usernames must be unique (verified via items dictionary [test: testDuplicatedUsername])
	- Emails must be unique (verified via emailsSet [test: testDuplicatedEmails])
	* Test class: UserTests!

UsersManager class
	instanceVariableNames: 'uniqueInstance'!

!UsersManager class methodsFor: 'as yet unclassified' stamp: 'afv 5/2/2013 20:04'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of UsersManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/10/2013 23:27'!
add: aUser
	"Adds a user to the structure.
	* Test class: UserTests
	"
	self validate: aUser.
	
	items add: (aUser username) -> aUser.
	emailsMap add: (aUser email) -> aUser.
	Transcript show: 'The user doesnt exists on the system, adding it...'; crlf.

! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/2/2013 18:01'!
asOrderedCollection
	"comment stating purpose of message"

	^ items asOrderedCollection.! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/11/2013 19:44'!
doLogin: aUsernameString password: aPasswordString
	"Authenticates the user into the system.
	Returns true if username/password matches the ones provided, false if not.
	Possible errors:
	- Username missmatch. Test: testDoLoginWithNonExistingUsername
	- Password missmatch. Test: testDoLoginWithWrongPassword
	- User is disabled. Test:
	"
	| user test acceptedAuth msg error |
	user := nil.
	test := items includesKey: aUsernameString.
	(test) ifFalse: [
		msg := 'User doesnt exist'.
		error := true.
	].
	(test) ifTrue: [
		user := items at: aUsernameString.
		error := false.
		acceptedAuth := user matchUsername: aUsernameString matchPassword: aPasswordString.
		acceptedAuth ifFalse: [
			msg := 'Wrong password. No cookies for you!!'.
			error := true.
		].
		acceptedAuth ifTrue: [
			acceptedAuth := acceptedAuth and: user isEnabled.
			acceptedAuth ifFalse: [
				msg := 'The user is disabled, you cannot login.'.
				error := true.
			].
			acceptedAuth ifTrue: [
				msg := 'User loged in'.
			].
		].
	].
	Transcript show: msg; cr.
	(error) ifTrue: [
		PermissionError signal: msg.
	].
	^ user.! !

!UsersManager methodsFor: 'initialize-release' stamp: 'afv 5/12/2013 17:08'!
initialize
	"Class responsability: handles several users and encapsulates authentication methods"
	
	super initialize.
	items := Dictionary new.
	emailsMap := Dictionary new.
	Transcript show: 'UserManager initialize'; crlf.! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/10/2013 10:47'!
items
	"Returns the internal structure"

	^ items.! !

!UsersManager methodsFor: 'initialize-release' stamp: 'it 5/11/2013 14:36'!
remove: aUsername
	"Removes from the structure aUsername. Raises errors on:
	- Non existing aUsername (verified with testEmptyUsername)"
	
	| user |
	Transcript show: 'Removing *', aUsername, '* from the database'; crlf.
	
	user := items at: aUsername ifAbsent: [
		self error: 'The user does not exists on the system, cannot remove it...'.
		^ self
	].

	items removeKey: aUsername.
	emailsMap removeKey: (user email) ifAbsent: [].
! !

!UsersManager methodsFor: 'accessing' stamp: 'afv 5/11/2013 12:34'!
removeAll
	"Internal method only. Not intended to be used as part of the main API"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].

	emailsMap keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		emailsMap removeKey: key.
	].! !

!UsersManager methodsFor: 'accessing' stamp: 'it 6/16/2013 11:32'!
save: aUserCopy
	"Saves a user to the structure"
	| user |
	
	(items includesKey: aUserCopy username) ifFalse: [
		ValidationError signal: 'The user you are trying to update does not exist' ].
	
	self validate: aUserCopy forExistingKey: aUserCopy username.
	
	user := items at: aUserCopy username.
	user copyFrom: aUserCopy.
	
	Transcript show: 'Updating existing user...'; crlf.
! !

!UsersManager methodsFor: 'accessing' stamp: 'it 5/11/2013 14:02'!
select: aFilterBlock
	"Performs a Collection select using the passed in argument. Returns the resulting collection."

	^ items select: aFilterBlock.
	! !

!UsersManager methodsFor: 'testing' stamp: 'it 6/16/2013 11:30'!
validate: aUser
	"Same as #validate:forExistingKey:# , but it does not take in the existingKey argument."
	self validate: aUser forExistingKey: nil  
	
	.
! !

!UsersManager methodsFor: 'testing' stamp: 'afv 6/17/2013 15:02'!
validate: aUser forExistingKey: existingUsername
	"Validates the passed-in User. If the User already exists in the database, the 'existingUsername' argument should be also informed; otherwise it should be nil.
	Raises errors on:
	- empty username (verified with testEmptyUsername)
	- empty password (verified with testEmptyPassword)
	- empty email (verified with testEmptyEmail)
	- empty role (verified with testEmptyRole)
	- empty status (verified with testEmptyStatus)
	- repeated usernames (verified via items dictionary [test: testDuplicatedUsername])
	- repeated emails (verified via emailsMap [test: testDuplicatedEmails])
	"
	
	(aUser username) isEmptyOrNil ifTrue: [
		ValidationError signal: 'Username cannot be empty!!'.
	].

	(aUser hasEmptyPassword) ifTrue: [
		ValidationError signal: 'Password cannot be empty Mr. genious!!'.
	].

	(aUser name) isEmptyOrNil ifTrue: [
		ValidationError signal: 'Name cannot be empty!!'.
	].

	(aUser email) isEmptyOrNil ifTrue: [
		ValidationError signal: 'Email cannot be empty!!'.
	].

	(aUser role) isEmptyOrNil ifTrue: [
		ValidationError signal: 'Role cannot be empty!!'.
	].

	(aUser status) isEmptyOrNil ifTrue: [
		ValidationError signal: 'Status cannot be empty oh oh!!'.
	].

	(existingUsername ~= nil) ifTrue: [	
		(emailsMap includesKey: (aUser email)) ifTrue: [
			"There is a User with the same email!!
			Checks if its not the same user. Raise an error only if it's a different user"
			(existingUsername  ~= ((emailsMap at: aUser email) username)) ifTrue: [
				ValidationError signal: 'A user with that email address already exists in the system!!'.
			]
		].

		(items includesKey: (aUser username)) ifTrue: [
			"There is a User with the same username!!
			Checks if its not the same user. Raise an error only if it's a different user"
			(existingUsername ~= aUser username) ifTrue: [
				ValidationError signal: 'Cannot add that username, it already exists on the system. Did you forget the password? No cookies for you!!'.
			].
		].
	]
! !

Error subclass: #PermissionError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!

Error subclass: #ValidationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Core'!
